"""Prisma Access SASE"""
from os.path import expanduser, exists
from getpass import getpass

import yaml

from prismasase.config import Auth, Config, set_bool

config = Config()
if not all([config.CLIENT_ID, config.CLIENT_SECRET, config.TSG]):
    home = expanduser("~")
    filename = f"{home}/.config/.prismasase"
    if exists(filename):
        with open(filename, 'r', encoding='utf-8') as yam:
            yaml_config = yaml.load(yam, Loader=yaml.FullLoader)
        config.CLIENT_ID = yaml_config['CLIENT_ID']
        config.CLIENT_SECRET = yaml_config['CLIENT_SECRET']
        config.TSG = yaml_config['TSG']
        config.CERT = yaml_config.get('CERT', False)
    else:
        config.CLIENT_ID = input("Please input Client ID: ")
        config.CLIENT_SECRET = getpass("Please input Client Secret: ")
        config.TSG = input("Please enter TSG ID: ")
        config.CERT = input("Please enter custom cert location" +
                            "('true'|'false'|<custom_cert_location>): ")
config.CERT = set_bool(config.CERT)
auth = Auth(tsg_id=config.TSG, client_id=config.CLIENT_ID,
            client_secret=config.CLIENT_SECRET, verify=config.CERT)



# pylint: disable=invalid-name
"""Generates a YAML Config File"""

from getpass import getpass
from os.path import exists, expanduser
from os import mkdir
import sys
import yaml


def gen_yaml():
    """Generates a prisma access SASE yaml config file

    Raises:
        ValueError: _description_
    """
    print('Running YAML Configs')
    CLIENT_ID = input("Please input Client ID: ")
    CLIENT_SECRET = getpass("Please input Client Secret: ")
    TSG = input("Please enter TSG ID: ")
    CERT = input("Please enter custom cert location" +
                 "('true'|'false'|<custom_cert_location>): ")
    if CERT.lower() in ['true', 'false']:
        CERT = CERT.lower()
    elif not exists(CERT):
        raise ValueError(f'{CERT} Does not exist')
    yaml_dict = {
        "TSG": TSG,
        "CLIENT_ID": CLIENT_ID,
        "CLIENT_SECRET": CLIENT_SECRET,
        "CERT": CERT
    }
    home = expanduser('~')
    base_dir = f"{home}/.config"
    if not exists(f"{home}/.config"):
        mkdir(base_dir)
    filename = f"{base_dir}/.prismasase"
    with open(rf'{filename}', 'w', encoding='utf-8') as yam:
        yaml.dump(yaml_dict, yam)
    print("Created YAML config file for Prisma Access SASE in user directory")
    sys.exit()


if __name__ == '__main__':
    gen_yaml()



# pylint: disable=no-member,invalid-name
"""configurations"""

import os
import time

import requests
import orjson

from prismasase.exceptions import SASEAuthError
from prismasase.statics import URL_BASE


def set_bool(value: str):
    """sets bool value when pulling string from os env

    Args:
        value (str): _description_

    Returns:
        (str|bool): String if certificate path is passed otherwise True|False
    """
    if isinstance(value, bool):
        pass
    elif str(value).lower() == 'true':
        value = True
    elif str(value).lower() == 'false':
        value = False
    else:
        value = False
    return value


class Config:
    """
    Configuration Utility
    """
    CERT = os.environ.get("CERT", False)
    TSG = os.environ.get("TSG", None)
    CLIENT_ID = os.environ.get("CLIENT_ID", None)
    CLIENT_SECRET = os.environ.get("CLIENT_SECRET", None)
    REST_API = {
        "bandwidth-allocations": f"{URL_BASE}/bandwidth-allocations",
        "ike-gateways": f"{URL_BASE}/ike-gateways",
        "ike-crypto-profiles": f"{URL_BASE}/ike-crypto-profiles",
        "ipsec-crypto-profiles": f"{URL_BASE}/ipsec-crypto-profiles",
        "ipsec-tunnels": f"{URL_BASE}/ipsec-tunnels",
        "remote-networks": f"{URL_BASE}/remote-networks",
        "config-version": f"{URL_BASE}/config-versions"
    }


class Auth:
    """Authorization to SASE API and refresh Decorator

    Raises:
        SASEAuthError: _description_

    Returns:
        _type_: _description_
    """
    TOKEN_URL = "https://auth.apps.paloaltonetworks.com/oauth2/access_token"

    def __init__(self, tsg_id: str, client_id: str, client_secret: str, **kwargs):
        """_summary_

        Args:
            tsg_id (str): _description_
            client_id (str): _description_
            client_secret (str): _description_
            verify (str|bool, optional): sets request to verify with a custom cert
             bypass verification or verify with standard library. Defaults to True
            timeout (int, optional): sets API call timeout. Defaults to 60
        """
        self.tsg_id = tsg_id
        self.client_id = client_id
        self.client_secret = client_secret
        self.verify = kwargs.get('verify', True)
        self.timeout: int = kwargs.get('timeout', 60)
        self.access_token_expiration = time.time()
        self.token = self.get_token()

    def get_token(self) -> str:
        """Get Bearer Token

        Raises:
            SASEAuthError: _description_

        Returns:
            str: _description_
        """
        url = self.TOKEN_URL
        headers = {"Content-Type": "application/x-www-form-urlencoded"}
        data = f"grant_type=client_credentials&scope=tsg_id:{self.tsg_id}"
        auth = (self.client_id, self.client_secret)
        response = requests.post(url=url, headers=headers, data=data,
                                 auth=auth, timeout=self.timeout, verify=self.verify)
        if response.status_code == 200:
            response = response.json()
            token = response['access_token']
            # set timmer to expire
            self.access_token_expiration = time.time() + response['expires_in']
        elif response.status_code == 401:
            raise SASEAuthError(orjson.dumps(response.json()).decode('utf-8'))
        else:
            response.raise_for_status()
        return token

    class Decorators():
        """Decorators

        Returns:
            _type_: _description_
        """
        @staticmethod
        def refresh_token(decorated):
            """refreshes token"""
            def wrapper(token, *args, **kwargs):
                if time.time() > token.access_token_expiration:
                    # regenerate token and reset timmer
                    token.get_token()
                # send back just token from auth class
                return decorated(token.token, *args, **kwargs)
            return wrapper



"""Rest Calls"""

import requests

from prismasase.config import Auth
from prismasase import config, auth


@auth.Decorators.refresh_token
def prisma_request(token: Auth, **kwargs):
    """_summary_

    Args:
        token (Auth): Auth class that is used to refresh bearer token upon expiration.
        url_type (str): specify the api call
        method (str): specifies the type of HTTPS method used
        params (dict, optional): specifies parameters passed to request
        data (str, optional): specifies the data being sent
        verify (str|bool, optional): sets request to verify with a custom
         cert bypass verification or verify with standard library. Defaults to True
        timeout (int, optional): sets API call timeout. Defaults to 60
        delete_object (str, required|optional): Required if method is DELETE
        put_object (str, required|optional): Required if method is PUT

    Returns:
        _type_: _description_
    """
    url_type: str = kwargs['url_type']
    method: str = kwargs['method'].Upper()
    params: dict = kwargs.get('params', None)
    url: str = config.REST_API[url_type]
    headers = {"authorization": f"Bearer {token}", "content-type": "application/json"}
    data: str = kwargs.get('data', None)
    verify = kwargs.get('verify', True)
    timeout: int = kwargs.get('timeout', 60)
    if method.lower() == 'delete':
        delete_object = kwargs['delete_object']
        url = f"{url}/{delete_object}"
    if method.lower() == 'put':
        put_object = kwargs['put_object']
        url = f"{url}/{put_object}"
    response = requests.request(method=method, url=url, headers=headers,
                                data=data, params=params, verify=verify, timeout=timeout)
    response.raise_for_status()
    return response.json()



# pylint: disable=exec-used
"""Set up"""

import os.path
from setuptools import setup, find_packages

__version__ = None

with open('requirements.txt', encoding='utf-8') as f:
    requirements = f.read().splitlines()
here = os.path.abspath(os.path.dirname(__file__))
exec(open(f"{here}/prismasase/_version.py", encoding='utf-8').read())
readme_path = os.path.join(here, "README.md")
if os.path.exists(readme_path):
    with open(readme_path, 'rb') as stream:
        readme = stream.read().decode('utf-8')

setup(
    name='prisma-access-sase',
    version=__version__,
    author="atav928",
    author_email="adam@tavnets.com",
    maintainer_email="adam@tavnets.com",
    packages=find_packages(),
    include_package_data=True,
    install_requires=requirements,
    test_suite="tests",
    url='https://github.com/atav928/prisma-access-sase',
    keywords=['sase', 'prisma access', 'prisma'],
    long_description=readme,
    long_description_content_type='text/markdown',
    entry_points={
        'console_scripts': [
            'prisma_yaml_script = prismasase.__main__:gen_yaml',
        ],
    },
)  # pragma: no cover
